@@ -23,6 +23,7 @@ import Data.Coerce (coerce)
 import Data.Foldable (for_)
 import Data.IORef (IORef, newIORef, readIORef, writeIORef)
 import Data.Kind (Type)
+import Data.Proxy (Proxy (Proxy))
 import Data.Type.Coercion (Coercion (Coercion))
 import GHC.Exts (Proxy#, proxy#)
 import System.IO.Unsafe (unsafePerformIO)
@@ -449,11 +450,11 @@ subsume2 i = cmp (bimap i (eq (# #))) (merge (# #))
 class (es1 :: Effects) :> (es2 :: Effects)
 
 -- | A set of effects @e@ is a subset of itself
-instance {-# INCOHERENT #-} e :> e
+instance e :> e
 
 -- | If @e@ is subset of @es@ then @e@ is a subset of a larger set, @x
 -- :& es@
-instance (e :> es) => e :> (x :& es)
+instance {-# INCOHERENT #-} (e :> es) => e :> (x :& es)
 
 -- Do we want this?
 -- instance {-# incoherent #-} (e :> es) => (e' :& e) :> (e' :> es)
@@ -461,7 +462,20 @@ instance (e :> es) => e :> (x :& es)
 -- This seems a bit wobbly
 
 -- | @e@ is a subset of a larger set @e :& es@
-instance {-# INCOHERENT #-} e :> (e :& es)
+instance e :> (e :& es)
+
+subset ::
+  forall e1 es m.
+  (Monad m) =>
+  (e1 :> es) =>
+  m ()
+subset = pure ()
+
+effTag :: Eff es (Proxy es)
+effTag = pure Proxy
+
+handleTag :: h e -> Eff es (Proxy e)
+handleTag _ = pure Proxy
 
 -- |
 -- @
@@ -792,7 +806,7 @@ handleCoroutine update finish f = do
 --
 -- @
 -- >>> runPureEff $ yieldToList $ \\y -> do
---       for_ [0 .. 4] $ \\i -> do
+--       forEach (inFoldable [0 .. 3]) $ \\i -> do
 --         yield y i
 --         yield y (i * 10)
 -- ([0, 0, 1, 10, 2, 20, 3, 30], ())
